import tkinter as tk
from tkinter import messagebox
import asyncio
import aiohttp
import yaml
import logging

# Configure logging
logging.basicConfig(level=logging.INFO, format='%(asctime)s - %(levelname)s - %(message)s')

# Load configuration
def load_config():
    with open('config.yaml', 'r', encoding='utf-8') as file:
        config = yaml.safe_load(file)
    return config

# Get API details
def get_api_details(config):
    api_details = config['trading_bot']['api']
    return api_details

# Fetch account information
async def fetch_account_info(api_details):
    base_url = api_details['base_url']
    app_key = api_details['app_key']
    app_secret = api_details['app_secret']
    account_number = api_details['account_number']

    headers = {
        'appKey': app_key,
        'appSecret': app_secret
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
            if response.status == 200:
                return await response.json()
            else:
                logging.error(f"계좌 정보 가져오기 오류: {response.status}")
                return None

# Update account information
async def update_account_info(api_details, label):
    while True:
        data = await fetch_account_info(api_details)
        if data:
            info = f"총 손익: {data['output'][0]['evlu_pfls_amt']}\n" \
                   f"총 매입: {data['output'][0]['pchs_amt_smtl']}\n" \
                   f"총 평가: {data['output'][0]['evlu_amt_smtl']}\n" \
                   f"실현손익: {data['output'][0]['real_evlu_pfls']}\n" \
                   f"추정자산: {data['output'][0]['nxtr_prvs_rcdl_pfls']}"
            label.config(text=info)
        await asyncio.sleep(5)  # Update every 5 seconds

# Fetch real-time data
async def fetch_real_time_data(api_details):
    base_url = api_details['base_url']
    app_key = api_details['app_key']
    app_secret = api_details['app_secret']

    headers = {
        'appKey': app_key,
        'appSecret': app_secret
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/quotations/inquire-price", headers=headers) as response:
            if response.status == 200:
                return await response.json()
            else:
                logging.error(f"데이터 가져오기 오류: {response.status}")
                return None

# Analyze data
async def analyze_data(data):
    logging.info("데이터 분석 중...")
    return {"signal": "buy", "symbol": "AAPL", "quantity": 10}

# Full sell
async def full_sell(api_details):
    base_url = api_details['base_url']
    app_key = api_details['app_key']
    app_secret = api_details['app_secret']
    account_number = api_details['account_number']

    headers = {
        'appKey': app_key,
        'appSecret': app_secret
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-balance", headers=headers) as response:
            if response.status != 200:
                logging.error(f"보유 자산 가져오기 오류: {response.status}")
                return

            holdings = await response.json()

        tasks = []
        for holding in holdings['output']:
            symbol = holding['pdno']
            quantity = holding['hldg_qty']

            order_payload = {
                'CANO': account_number,
                'ACNT_PRDT_CD': '01',
                'PDNO': symbol,
                'ORD_DVSN': '00',
                'ORD_QTY': quantity,
                'ORD_UNPR': '0'
            }

            tasks.append(place_order(session, base_url, headers, order_payload))

        await asyncio.gather(*tasks)

async def place_order(session, base_url, headers, order_payload):
    async with session.post(f"{base_url}/uapi/domestic-stock/v1/trading/order-cash", json=order_payload, headers=headers) as response:
        if response.status == 200:
            logging.info(f"{order_payload['PDNO']} 매도 주문 성공")
        else:
            logging.error(f"{order_payload['PDNO']} 매도 주문 실패: {response.status}")

# Cancel all orders
async def cancel_all_orders(api_details):
    base_url = api_details['base_url']
    app_key = api_details['app_key']
    app_secret = api_details['app_secret']

    headers = {
        'appKey': app_key,
        'appSecret': app_secret
    }

    async with aiohttp.ClientSession() as session:
        async with session.get(f"{base_url}/uapi/domestic-stock/v1/trading/inquire-unsettled", headers=headers) as response:
            if response.status != 200:
                logging.error(f"주문 조회 오류: {response.status}")
                return

            orders = await response.json()

        tasks = []
        for order in orders['output']:
            order_id = order['ODNO']
            tasks.append(cancel_order(session, base_url, headers, order_id))

        await asyncio.gather(*tasks)

async def cancel_order(session, base_url, headers, order_id):
    async with session.delete(f"{base_url}/uapi/domestic-stock/v1/trading/order-cancel/{order_id}", headers=headers) as response:
        if response.status == 200:
            logging.info(f"주문 {order_id} 취소 성공")
        else:
            logging.error(f"주문 {order_id} 취소 실패: {response.status}")

# Cancel selected order
async def cancel_selected_order(api_details, order_id):
    base_url = api_details['base_url']
    app_key = api_details['app_key']
    app_secret = api_details['app_secret']

    headers = {
        'appKey': app_key,
        'appSecret': app_secret
    }

    async with aiohttp.ClientSession() as session:
        async with session.delete(f"{base_url}/uapi/domestic-stock/v1/trading/order-cancel/{order_id}", headers=headers) as response:
            if response.status == 200:
                logging.info(f"주문 {order_id} 취소 성공")
            else:
                logging.error(f"주문 {order_id} 취소 실패: {response.status}")

# Run actions based on user input
def run_action(action, order_id=None):
    config = load_config()
    api_details = get_api_details(config)
    
    loop = asyncio.get_event_loop()
    
    if action == 'fetch_data':
        data = loop.run_until_complete(fetch_real_time_data(api_details))
        if data:
            messagebox.showinfo("데이터 가져오기", str(data))
    elif action == 'analyze':
        data = loop.run_until_complete(fetch_real_time_data(api_details))
        if data:
            analysis_result = loop.run_until_complete(analyze_data(data))
            messagebox.showinfo("데이터 분석", str(analysis_result))
    elif action == 'full_sell':
        loop.run_until_complete(full_sell(api_details))
        messagebox.showinfo("전체 매도", "전체 매도 실행 완료")
    elif action == 'cancel_all':
        loop.run_until_complete(cancel_all_orders(api_details))
        messagebox.showinfo("모든 주문 취소", "모든 주문 취소 완료")
    elif action == 'cancel_order':
        if not order_id:
            messagebox.showerror("오류", "주문 ID가 필요합니다.")
            return
        loop.run_until_complete(cancel_selected_order(api_details, order_id))
        messagebox.showinfo("주문 취소", f"주문 {order_id} 취소 완료")
    else:
        messagebox.showerror("오류", "잘못된 동작입니다.")

def create_gui():
    root = tk.Tk()
    root.title("트레이딩 봇")

    # Set larger window size
    root.geometry("800x600")

    # Create a frame with padding around the elements
    frame = tk.Frame(root, padx=20, pady=20)
    frame.grid(row=0, column=0, padx=20, pady=20)

    tk.Label(frame, text="동작을 선택하세요:", font=("Arial", 16)).grid(row=0, column=0, columnspan=2, sticky='w', pady=10)

    # Adding buttons with colors based on their purpose
    tk.Button(frame, text="데이터 가져오기", command=lambda: run_action('fetch_data'), bg='lightblue', font=("Arial", 14)).grid(row=1, column=0, columnspan=2, sticky='ew', pady=5)
    tk.Button(frame, text="데이터 분석", command=lambda: run_action('analyze'), bg='lightgreen', font=("Arial", 14)).grid(row=2, column=0, columnspan=2, sticky='ew', pady=5)
    tk.Button(frame, text="전체 매도", command=lambda: run_action('full_sell'), bg='red', fg='white', font=("Arial", 14)).grid(row=3, column=0, columnspan=2, sticky='ew', pady=5)
    tk.Button(frame, text="모든 주문 취소", command=lambda: run_action('cancel_all'), bg='orange', font=("Arial", 14)).grid(row=4, column=0, columnspan=2, sticky='ew', pady=5)

    tk.Label(frame, text="취소할 주문 ID:", font=("Arial", 14)).grid(row=5, column=0, sticky='e', pady=10)
    order_id_entry = tk.Entry(frame, font=("Arial", 14))
    order_id_entry.grid(row=5, column=1, sticky='ew', pady=10)

    tk.Button(frame, text="주문 취소", command=lambda: run_action('cancel_order', order_id_entry.get()), bg='yellow', font=("Arial", 14)).grid(row=6, column=0, columnspan=2, sticky='ew', pady=5)

    tk.Label(frame, text="계좌 정보:", font=("Arial", 16)).grid(row=7, column=0, columnspan=2, sticky='w', pady=10)
    account_info_label = tk.Label(frame, text="정보 없음", justify='left', font=("Arial", 14))
    account_info_label.grid(row=8, column=0, columnspan=2, sticky='w', pady=10)

    # Make the second column expand with the window
    frame.columnconfigure(1, weight=1)

    # Update account info based on selected mode
    config = load_config()
    api_details = get_api_details(config)
    asyncio.create_task(update_account_info(api_details, account_info_label))

    root.mainloop()

if __name__ == "__main__":
    create_gui()
